Method: AE::Legacy::Assertions#assert_equal

Defined in:
lib/ae/legacy.rb

#assert_equal(exp, act, msg = nil) ⇒ Object

Passes if expected == +actual.

Note that the ordering of arguments is important, since a helpful error message is generated when this one fails that tells you the values of expected and actual.

Examples:

assert_equal 'MY STRING', 'my string'.upcase

Returns:

  • nothing

Raises:



72
73
74
75
76
# File 'lib/ae/legacy.rb', line 72

def assert_equal(exp, act, msg=nil)
  test = (exp == act)
  msg  = "Expected #{act.inspect} to be equal to #{exp.inspect}" unless msg
  __assert__(test, msg)
end