Module: Test
- Defined in:
- lib/ae/adapters/testunit.rb
Overview
TestUnit uses #add_assertion on it’s result object to track counts. We capture the result object by overriding the TestCase#run method, store it in a global variable and then use it when AE increments assertion counts.
Defined Under Namespace
Modules: Unit